﻿using Infragistics.Windows.DataPresenter;
using Infragistics.Windows.DataPresenter.Events;
using Infragistics.Windows.Editors;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace EditableObjects
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        // ##### Method 1: PREVENTIVE approach (see Readme.txt) #####
        private void XamDataGrid_EditModeEnding(object sender, EditModeEndingEventArgs e)
        {
            if (!e.Force)
            {               
                bool hasDataError = HasDataError(e.Cell.Record);

                if (hasDataError)
                    e.Cancel = true;
            }
        }

        // ##### Method 2: CORRECTIVE approach (see Readme.txt) #####
        private void XamDataGrid_EditModeEnded(object sender, EditModeEndedEventArgs e)
        {
            bool hasDataError = HasDataError(e.Cell.Record);

            if (hasDataError)
                ((XamDataGrid)sender).ExecuteCommand(DataPresenterCommands.StartEditMode);
        }

        /// <summary>Checks whether the row is invalid.</summary>
        private bool HasDataError(DataRecord dataRecord)
        {
            bool recordHasError = dataRecord.HasDataError;
            bool objectHasError = !string.IsNullOrEmpty(((IDataErrorInfo)dataRecord.DataItem).Error);

            return recordHasError || objectHasError;
        }

        private void XamDataGrid_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            // Pressing ESCAPE rolls back the entire row.
            if (e.Key == Key.Escape)
            {
                ((XamDataGrid)sender).ExecuteCommand(DataPresenterCommands.DiscardChangesToActiveRecord);
                ((XamDataGrid)sender).ExecuteCommand(DataPresenterCommands.EndEditModeAndDiscardChanges);
                e.Handled = true;
            }
        }

        private void XamDataGrid_RecordUpdated(object sender, RecordUpdatedEventArgs e)
        {
            if (e.Record.IsDataRecord)
            {
                bool hasErrors = HasDataError(e.Record);

                if (!hasErrors)
                {
                    DataObject dataObject = (DataObject)e.Record.DataItem;
                    Trace.WriteLine(string.Format("Saving changes to database: {0} -> {1}.", dataObject.Name, dataObject.Icon));
                }
            }
        }
    }
}
